from unittest import TestCase
from file_readers.excel.excel_data_collector import ExcelDataCollector
from openpyxl.cell import Cell


class SimpleCell(object):
    # TODO: Figure out how to use openpyxl Cell objects instead. Could not create a Cell object with a value..
    def __init__(self, coordinate, value):
        self.coordinate = coordinate
        self.value = value


class TestExcelDataCollector(TestCase):

    def test_collect_data(self):
        data_source = []
        row = (SimpleCell("A1", "Name"),
               SimpleCell("B1", "Age"),
               SimpleCell("C1", "Title"))
        data_source.append(row)

        row = (SimpleCell("A2", "John"),
               SimpleCell("B2", 24),
               SimpleCell("C2", "Developer"))
        data_source.append(row)

        collector = ExcelDataCollector()
        for row in data_source:
            collector.collect_data(row)

        result_row = collector.data[0]
        self.assertEqual(result_row['Name'], 'John')
        self.assertEqual(result_row['Age'], 24)
        self.assertEqual(result_row['Title'], "Developer")



import csv
from file_readers.file_reader import BaseFileReader
from file_readers.data_collector import BaseDataCollector


class CSVFileReader(BaseFileReader):

    def __init__(self, data_collector, delimiter, quotechar=None, debug=False):
        """
        Args:
            data_collector: A data collector which is a subclass of BaseDataCollector.
            delimiter: Delimiter to be used when reading the CSV file.
            quotechar:
            debug: If set to True, then each row is printed out to stdout.


        Raises:
            AttributeError: Provided data collector is not a subclass of BaseDataCollector.
        """
        if not isinstance(data_collector, BaseDataCollector):
            raise AttributeError('Provided data collector is not a subclass of BaseDataCollector!')
        self.data_collector = data_collector
        self.delimiter = delimiter
        self.quotechar = quotechar
        self.debug = debug

    def read_file(self, filename):
        """Reads the given file and collects data from each row.

        Args:
            filename: name of the file to read, including the path.
        """
        with open(filename, 'r', newline='') as csv_file:
            if self.quotechar:
                reader = csv.reader(csv_file, delimiter=self.delimiter, quotechar=self.quotechar)
            else:
                reader = csv.reader(csv_file, delimiter=self.delimiter)
            for row in reader:
                if self.debug:
                    print(row)
                self.data_collector.collect_data(row)

    @staticmethod
    def read_data_from_file(filename, func, header=None, delimiter=' ', quotechar='|', debug=False):
        """Reads the given file and collects data from each row.

            Args:
                filename: name of the file to read, including the path.
                func: A function that collects the data into a list.
                header: The header of the first column. Default: None
                delimiter: Delimiter to be used when reading the CSV file.
                quotechar:
                debug: If set to True, then each row is printed out to stdout.

            Returns:
                A list with items added by func.
            """
        data = []
        with open(filename, 'r', newline='') as csvfile:
            reader = csv.reader(csvfile, delimiter=delimiter, quotechar=quotechar)
            for row in reader:
                if debug:
                    print(row)
                if header and header in row[0]:
                    continue
                func(data, row)
        return data



from unittest import TestCase
import os
from file_readers.csv.csv_filereader import CSVFileReader
from file_readers.data_collector import BaseDataCollector

base_path = os.path.dirname(__file__)


class DummyDataCollector(BaseDataCollector):
    def __init__(self):
        self.data_structure = [
            {'header': 'name', 'index': 0, 'type': str},
            {'header': 'age', 'index': 1, 'type': int},
            {'header': 'title', 'index': 2, 'type': str}
        ]
        self.data = []

    def collect_data(self, source):
        values = {}
        for item in self.data_structure:
            header = item['header']
            index = item['index']
            values[header] = source[index]
        self.data.append(values)


def data_collector(data, row):
    values = {'name': row[0], 'age': int(row[1]), 'title': row[2]}
    data.append(values)


class TestCSVFileReader(TestCase):

    def test_read_file(self):
        test_file = base_path + "/data/test.csv"
        collector = DummyDataCollector()
        reader = CSVFileReader(data_collector=collector, delimiter=";")
        reader.read_file(test_file)
        row = collector.data[1]
        self.assertEqual(row['name'], 'John')
        self.assertEqual(int(row['age']), 24)
        self.assertEqual(row['title'], 'Developer')

    def test_read_data_from_file(self):
        test_file = base_path + "/data/test.csv"
        data = CSVFileReader.read_data_from_file(test_file, func=data_collector, header='Name', delimiter=";")
        row = data[0]
        self.assertEqual(row['name'], 'John')
        self.assertEqual(row['age'], 24)
        self.assertEqual(row['title'], 'Developer')



from file_readers.data_collector import BaseDataCollector


class CSVRowDataCollector(BaseDataCollector):

    def __init__(self, data_structure_dict, header=None):
        """The constructor takes the data sources structure as parameter.

        Args:
            data_structure_dict: The structure of the data source to collect.
                Example structure:
                    data_structure = [
                        {'header': 'name', 'index': 0, 'type': str},
                        {'header': 'age', 'index': 1, 'type': int},
                        {'header': 'title', 'index': 2, 'type': str}
                    ]
            header: The header of the first column.
        """
        self.data_structure = data_structure_dict
        self.header = header
        self.data = []

    def collect_data(self, source):
        """Collects the data from a row data source.

        The data source must be a list containing the data.
        """
        if self.header and self.header in source[0]:  # This header check might be misleading
            return
        values = {}
        for item in self.data_structure:
            header = item['header']
            index = item['index']
            values[header] = source[index]

        self.data.append(values)



from file_readers.data_collector import BaseDataCollector
from openpyxl.cell import column_index_from_string


class ExcelDataCollector(BaseDataCollector):

    def __init__(self, debug=False):
        self.debug = debug
        self.data = []
        self.headers = []

    def collect_data(self, source):
        """Collects all data from a sheet

        Args:
            source: A list or tuple of Cell objects
        """

        row = {}
        for cellObj in source:
            if "1" in cellObj.coordinate:
                self.headers.append(cellObj.value)
                continue

            index = column_index_from_string(cellObj.coordinate[0]) - 1
            header = self.headers[index]
            row[header] = cellObj.value
        if self.debug:
            print(row)
        if row:
            self.data.append(row)



NAME = 'file-readers'
VERSION = '0.0.3'
DESCRIPTION = 'Library of different types of file reader and data collector classes.'
URL = 'https://github.com/mjalas/file-readers'
AUTHOR = 'Mats Jalas'
AUTHOR_EMAIL = 'mats.jalas@gmail.com'
LICENSE = 'MIT'



from file_readers.file_reader import BaseFileReader
from file_readers.excel.excel_data_collector import ExcelDataCollector
from file_readers.excel.excel_reader import ExcelReader


class DefaultExcelReader(BaseFileReader):
    """A excel file reader that uses the default
    excel data collector.
    """

    def __init__(self):
        self.data_collector = ExcelDataCollector()
        self.reader = ExcelReader(self.data_collector)
        self.data = []
        self.sheet_to_read = None

    def set_sheet_to_read(self, sheet_name):
        self.sheet_to_read = sheet_name

    def read_file(self, filename):
        self.reader.set_sheet_to_read(self.sheet_to_read)
        self.reader.read_file(filename)
        self.data = self.collector.data

    def get_data(self):
        return self.data



from file_readers.file_reader import BaseFileReader
from file_readers.data_collector import BaseDataCollector
from openpyxl import load_workbook
from openpyxl.cell import get_column_letter


class ExcelReader(BaseFileReader):

    def __init__(self, data_collector, debug=False):
        if not isinstance(data_collector, BaseDataCollector):
            raise AttributeError("Provided data collector is not a subclass of BaseDataCollector!")
        self.data_collector = data_collector
        self.debug = debug
        self.sheet_to_read = None

    def set_sheet_to_read(self, sheet_name):
        self.sheet_to_read = sheet_name

    @staticmethod
    def get_sheet_names(filename):
        work_book = load_workbook(filename)
        return work_book.get_sheet_names()

    def read_file(self, filename):
        work_book = load_workbook(filename)
        if self.sheet_to_read is None:
            self.sheet_to_read = work_book.get_active_sheet()
        sheet = work_book.get_sheet_by_name(self.sheet_to_read)
        rows = sheet.max_row
        columns = sheet.max_column
        min = 'A1'
        max_letter = get_column_letter(columns)
        max = max_letter + str(rows)

        for rowOfCellObjects in sheet[min:max]:
            self.data_collector.collect_data(rowOfCellObjects)



from unittest import TestCase

from file_readers.csv.csv_row_data_collector import CSVRowDataCollector


class TestCSVRowDataCollector(TestCase):

    def test_collect_data(self):
        data_structure = [
            {'header': 'name', 'index': 0, 'type': str},
            {'header': 'age', 'index': 1, 'type': int},
            {'header': 'title', 'index': 2, 'type': str}
        ]
        collector = CSVRowDataCollector(data_structure_dict=data_structure)
        row = ['John', 24, 'Developer']
        collector.collect_data(row)
        data = collector.data[0]
        self.assertEqual(data['name'], 'John')
        self.assertEqual(data['age'], 24)
        self.assertEqual(data['title'], 'Developer')



from unittest import TestCase
import os
from file_readers.excel.excel_reader import ExcelReader
from file_readers.excel.excel_data_collector import ExcelDataCollector

base_path = os.path.dirname(__file__)


class TestExcelReader(TestCase):

    def test_get_sheet_names(self):
        test_file = base_path + "/data/test.xlsx"
        sheets = ExcelReader.get_sheet_names(test_file)
        self.assertEqual('Personnel', sheets[0])

    def test_read_file(self):
        test_file = base_path + "/data/test.xlsx"
        collector = ExcelDataCollector()
        reader = ExcelReader(data_collector=collector)
        reader.set_sheet_to_read("Personnel")
        reader.read_file(test_file)
        result_row = collector.data[0]
        self.assertEqual(result_row['Name'], 'John')
        self.assertEqual(result_row['Age'], 24)
        self.assertEqual(result_row['Title'], "Developer")



from setuptools import setup, find_packages
from codecs import open
from os import path
import file_readers.metadata as metadata

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
# TODO: Currently not working because of markdown to reStructuredText conversion not added.
long_description = ""
if path.exists(path.join(here, 'README.rst')):
    with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
        long_description = f.read()

setup(
    name=metadata.NAME,
    version=metadata.VERSION,
    description=metadata.DESCRIPTION,
    long_description=long_description,
    url=metadata.URL,
    author=metadata.AUTHOR,
    author_email=metadata.AUTHOR_EMAIL,
    license=metadata.LICENSE,
     # See https://PyPI.python.org/PyPI?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.5',
    ],
    packages=find_packages(exclude=['csv/tests', 'excel/tests']),
    setup_requires=['nose>=1.0', 'coverage>=4.0.3', 'openpyxl>=2.3.4'],
    test_suite='nose.collector',
    tests_require=['nose'],
    install_requires=['openpyxl'],
)
